function [] = path_tracing_zeta(start_options,pars,bifurcation_mode)
% perform path tracing over zeta to find solutions of the LLE for f_1 = 0.
% 
% If bifurcation_mode = 0, then the path of constant solutions is
% computed. The start is then determined by the fields zeta_start & 
% start_direction in start_options. The field start_direction determines 
% if zeta increases (+1) or decreases (-1) at the beginning

% If bifurcation_mode = 1, then a bifurcation branch of the one mode LLE is
% computed. The start is then determined by the fields bc_mat, zeta and 
% tangent_vector in start_options. Further fields that have to be taken 
% into account are direction, level (1/2/... for primary/secondary/... 
% bifurcation)

% If bifurcation_mode = 2, then a (constant) solution for a specific value 
% of zeta is sought-after as a preparation for path tracing w.r.t. f_1. The
% structure start_options thus contains a field 'zeta'. Further, it can
% contain a field 'level' which determines which of mutliple solutions for
% the respective zeta values shall be used (level = 1 --> solution with
% highest norm etc.)

%% Preparations
% Lese Level des zu berechnenden Pfades aus (constant solution/primary
% bifurcation/...) --> Nötig für Foldername
if bifurcation_mode == 0
    level = 0;
else
    level = start_options.level;
end
% Erstelle Ordner zur Speicherung des Pfades
[fld,filename_vecs] = create_folder(pars,bifurcation_mode,level);
% Überprüfe, ob der Pfad schon vollstänig berechnet wurde
if exist(filename_vecs,'file')>0
    load(filename_vecs,'path_completed')
    if path_completed
        disp('Path has been computed completely before')
        return
    end
end
% Create or load data structures for bifurcation points
[bifurcation_points,bifurcation_points_todo,filename_bifur,filename_bifur_todo]...
    = create_bifurcation_structures(pars);
% Schrittzähler
k = 0; 
% Initialisiere Plot
handles = initialize_path_tracing_plot('zeta',pars,true,false);
drawnow()

%% Initialize path tracing
if bifurcation_mode == 0 % Finde konstante Lösungen der Ein-Moden-LLE
    % Startpunkt berechnen mit fsolve
    [zeta,bc_mat,direction,tangent_vector] = find_start_solution(start_options,1,1,pars);
    disp('Startpunkt mit fsolve berechnet')
    % Weitere Vorbereitungen
    branch = 'top';
elseif bifurcation_mode == 1 % berechne einen Verzweigungspfad der Ein-Moden-LLE
    % Startpunkt auslesen
    zeta = start_options.zeta;
    bc_mat = start_options.bc_mat;
    direction = start_options.direction;
    point_vecs_try = add_point_to_path_vecs(zeta,bc_mat,0,direction,[],pars);
    % Startpunkt in Plot eintragen
    handles = add_point_to_path_plot(handles,point_vecs_try,'zeta');
    handles = draw_current_solution(handles,bc_mat,pars,true);
    drawnow()
    % Teste, ob Pfadverfolgung mit dem angegebenen Tangentenvektor
    % funktioniert:
    tangent_vector = start_options.tangent_vector;
    [success,direction] = try_bifurcation(tangent_vector,bc_mat,zeta,direction,pars);
    if success == false
        % Diese Verzweigung funktioniert nicht! Beenden!
        return
    end
    branch = '';
end
% Speichere Startpunkt und trage ihn in Plot ein
point_vecs = save_new_point(zeta,bc_mat,direction,[],filename_vecs,k,branch,fld,pars);
handles = add_point_to_path_plot(handles,point_vecs,'zeta');
handles = draw_current_solution(handles,bc_mat,pars,true);
drawnow()

%% path tracing
continue_tracing = true; % Abbruchkriterium
msg_length = 0; % Um die vorherige Ausgabe im Command Window zu entfernen
approximation_mode = false; % Modus, in dem wir uns einem Verzweigungspunkt annähren
lambda = pars.lambda_0;
while continue_tracing
    k = k + 1;
    %% berechne nächsten Punkt (Prädiktor + Korrrektorschritte)
    [bc_mat_new,zeta_new,tangent_vector_new] ...
        = path_tracing_step(bc_mat,zeta,tangent_vector,lambda,direction,pars);

    %% Kommt man einem Verzweigungspunkt nahe? 
    % Falls ja, in entsprechenden Modus wechseln und kleinere Schrittweite verwenden
    if approximation_mode == false && sum(tangent_vector.*tangent_vector_new)<0  
        disp('Pfad nähert sich einem Verzweigungspunkt. Schrittweite wird reduziert.')
        approximation_mode = true;
        lambda = pars.lambda_0/16;
        [bc_mat_new,zeta_new,tangent_vector_new] ...
            = path_tracing_step(bc_mat,zeta,tangent_vector,lambda,direction,pars);
        msg_length = 0;
    end
    %% Verzweigungspunkt übersprungen?
    % Falls nächster Tangentenvektor entgegen letztem Tangentenvektor
    % (Überprüfung durch SKP der Vektoren):
    if sum(tangent_vector.*tangent_vector_new)<0
        % Verzweigungspunkt übersprungen! Löse ihn auf.
        disp('Verzweigungspunkt erreicht! Wird nun aufgelöst.')
        [bc_bifur,zeta_bifur,lambda_total,k,point_vecs,handles] = ...
            resolve_bifurcation_point(bc_mat,zeta,tangent_vector,lambda,direction,k,branch,point_vecs,filename_vecs,fld,pars,handles);
        % Überprüfe, ob der Verzweigungspunkt einem vorherigen
        % Verzweigungspunkt entspricht
        [is_new,~] = is_new_bifurcation_point(bc_bifur,zeta_bifur,bifurcation_points,pars);
        if is_new
            disp('Es handelt sich um einen neuen Verzweigungspunkt.')
            % speichere ihn in den bifurcation_points structures 
            [bifurcation_points,bifurcation_points_todo] = ...
                save_new_bifurcation_point(zeta_bifur,bc_bifur,level,zeta,bc_mat,...
                    bifurcation_points,bifurcation_points_todo,...
                    filename_bifur,filename_bifur_todo,pars,bifurcation_mode);
            % Ergänze Verzweigungspunkt im Plot
            handles = add_bifurcation_point(handles,bifurcation_points);
            drawnow()
        else
            disp('Es handelt sich um einen bekannten Verzweigungspunkt.')
            % Ggf. braucht der entsprechende Tangentenvektor aus der
            % bifurcation_points_todo structure nicht mehr betrachtet
            % werden! --> Untersuchen!
            bifurcation_points_todo ...
                = delete_tangent_vector(zeta_bifur,bc_bifur,zeta,bc_mat,...
                              bifurcation_points_todo,filename_bifur_todo,pars);
        end
        % Falls aktuell ein Verzweigungspfad berechnet wird: Ende des
        % Abschnitts erreicht, anhalten.
        if bifurcation_mode == 1 % Ende des Verzweigungspfades erreicht, fertig
            break
        end
        % Falls aktuell der Pfad konstanter Lösungen berechnet wird:
        % Mache mit Pfadverfolgung kurz hinter dem Verzweigungspunkt
        % weiter.
        [bc_mat_new,zeta_new,tangent_vector_new] ...
            = path_tracing_step(bc_mat,zeta,tangent_vector,lambda_total+1e-3,direction,pars);
        % Ersetze Tangentenvektor durch -tangentenvektor, direction durch
        % -direction, gehe wieder in normalen Modus über
        tangent_vector_new = - tangent_vector_new;
        direction = - direction;
        approximation_mode = false;
        lambda = pars.lambda_0;
        msg_length = 0;
    end

    %% Wendepunkt übersprungen? --> Definiere neuen Abschnitt auf dem Pfad
    if bifurcation_mode == 0 
        [branch,new_branch] = update_branch(zeta,zeta_new,branch);
        if new_branch
            disp('Wendepunkt passiert')
            msg_length = 0;
        end
    end

    %% Speichere neuen Punkt
    msg = [num2str(k),'-ten Punkt auf Pfad berechnet, zeta = ',num2str(zeta_new),'\n'];
    fprintf([ repmat(sprintf('\b'), 1, msg_length-1) , msg ]); % Löscht vorherige Nachricht und gibt neue aus
    if mod(k,25)==0
        msg_length = 0; % Alle 25 Schritte Ergebnisse in neue Zeile schreiben
    else
        msg_length = length(msg);
    end
    point_vecs = save_new_point(zeta_new,bc_mat_new,direction,...
                                point_vecs,filename_vecs,k,branch,fld,pars);
    %% Aktualisiere Daten im Plot
    handles = add_point_to_path_plot(handles,point_vecs,'zeta');
    handles = draw_current_solution(handles,bc_mat_new,pars,true);
    drawnow()

    %% Bereite nächsten Schritt vor
    tangent_vector = tangent_vector_new;
    bc_mat = bc_mat_new; zeta = zeta_new;

    
    %% Abbruch falls zeta zu groß/klein, sonst weiter rechnen
    if zeta > 7 || zeta < -2
        continue_tracing = false;
    end
end

% path has been computed completely --> update the variable path_completed
path_completed = true;
save(filename_vecs,'point_vecs','path_completed')

disp('Path computed completely')
end

